# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - EventAggregatorNewEvent Action

    @copyright: 2008, 2009, 2010 by Paul Boddie <paul@boddie.org.uk>
    @copyright: 2000-2004 Juergen Hermann <jh@web.de>,
                2003-2008 MoinMoin:ThomasWaldmann,
                2004-2006 MoinMoin:AlexanderSchremmer,
                2007 MoinMoin:ReimarBauer.
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from MoinMoin.action import ActionBase
from MoinMoin.Page import Page
from MoinMoin.PageEditor import PageEditor
import EventAggregatorSupport
import calendar
import re

Dependencies = ['pages']

# Action class and supporting functions.

class EventAggregatorNewEvent(ActionBase):

    "An event creation dialogue requesting various parameters."

    def _get_selected(self, value, input_value):
        return input_value is not None and value == input_value and 'selected="selected"' or ''

    def _get_selected_for_list(self, value, input_values):
        return value in input_values and 'selected="selected"' or ''

    def _get_input(self, form, name, default):
        return int(form.get(name, [None])[0] or default)

    def get_form_html(self, buttons_html):
        _ = self._
        request = self.request
        form = request.form

        # Handle advanced and basic forms.

        show_advanced = form.get("advanced") and not form.get("basic")

        # Prepare the category list.

        category_list = []
        category_pagenames = form.get("category", [])

        for category_name, category_pagename in \
            EventAggregatorSupport.getCategoryMapping(
                EventAggregatorSupport.getCategories(request),
                request):

            selected = self._get_selected_for_list(category_pagename, category_pagenames)

            # In the advanced view, populate a menu.

            if show_advanced:
                category_list.append('<option value="%s" %s>%s</option>' % (category_pagename, selected, category_name))

            # In the basic view, use hidden fields.

            elif selected:
                category_list.append('<input value="%s" name="category" type="hidden" />' % category_pagename)

        # Initialise month lists.

        start_month_list = []
        start_month_list.append('<option value=""></option>')
        end_month_list = []
        end_month_list.append('<option value=""></option>')

        start_month = self._get_input(form, "start-month", 0)
        end_month = self._get_input(form, "end-month", 0)

        # Prepare month lists, selecting specified months.

        for month in range(1, 13):
            month_label = _(EventAggregatorSupport.getMonthLabel(month))
            selected = self._get_selected(month, start_month)
            start_month_list.append('<option value="%02d" %s>%s</option>' % (month, selected, month_label))
            selected = self._get_selected(month, end_month)
            end_month_list.append('<option value="%02d" %s>%s</option>' % (month, selected, month_label))

        # Permitting configuration of the template name.

        template_default = getattr(request.cfg, "event_aggregator_new_event_template", "EventTemplate")

        d = {
            "buttons_html"          : buttons_html,
            "category_label"        : _("Categories"),
            "category_list"         : "\n".join(category_list),
            "start_month_list"      : "\n".join(start_month_list),
            "end_month_list"        : "\n".join(end_month_list),
            "start_label"           : _("Start date (day, month, year)"),
            "start_day_default"     : form.get("start-day", [""])[0],
            "start_year_default"    : form.get("start-year", [""])[0] or EventAggregatorSupport.getCurrentYear(),
            "end_label"             : _("End date (day, month, year) - if different"),
            "end_day_default"       : form.get("end-day", [""])[0],
            "end_year_default"      : form.get("end-year", [""])[0],
            "title_label"           : _("Event title/summary"),
            "title_default"         : form.get("title", [""])[0],
            "description_label"     : _("Event description"),
            "description_default"   : form.get("description", [""])[0],
            "template_label"        : _("Event template"),
            "template_default"      : form.get("template", [""])[0] or template_default,
            "parent_label"          : _("Parent page"),
            "parent_default"        : form.get("parent", [""])[0],
            "advanced_label"        : _("Show advanced options"),
            "basic_label"           : _("Show basic options"),
            }

        # Prepare the output HTML.

        html = '''
<table>
    <tr>
        <td class="label"><label>%(title_label)s</label></td>
        <td>
            <input name="title" type="text" size="40" value="%(title_default)s" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(start_label)s</label></td>
        <td>
            <input name="start-day" type="text" value="%(start_day_default)s" size="2" />
            <select name="start-month">
                %(start_month_list)s
            </select>
            <input name="start-year" type="text" value="%(start_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(end_label)s</label></td>
        <td>
            <input name="end-day" type="text" value="%(end_day_default)s" size="2" />
            <select name="end-month">
                %(end_month_list)s
            </select>
            <input name="end-year" type="text" value="%(end_year_default)s" size="4" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(description_label)s</label></td>
        <td>
            <input name="description" type="text" size="40" value="%(description_default)s" />
        </td>
    </tr>''' % d

        if show_advanced:
            html += '''
    <tr>
        <td></td>
        <td>
            <input name="basic" type="submit" value="%(basic_label)s" />
            <input name="advanced" type="hidden" value="true" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(category_label)s</label></td>
        <td class="content">
            <select multiple="multiple" name="category">
                %(category_list)s
            </select>
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(template_label)s</label></td>
        <td>
            <input name="template" type="text" size="40" value="%(template_default)s" />
        </td>
    </tr>
    <tr>
        <td class="label"><label>%(parent_label)s</label></td>
        <td>
            <input name="parent" type="text" size="40" value="%(parent_default)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d
        else:
            html += '''
    <tr>
        <td></td>
        <td>
            <input name="advanced" type="submit" value="%(advanced_label)s" />
            %(category_list)s
            <input name="parent" type="hidden" value="%(parent_default)s" />
            <input name="template" type="hidden" value="%(template_default)s" />
        </td>
    </tr>
    <tr>
        <td></td>
        <td class="buttons">
            %(buttons_html)s
        </td>
    </tr>
</table>''' % d

        return html

    def do_action(self):

        "Create the new event."

        _ = self._
        form = self.request.form

        # If no title exists in the request, an error message is returned.

        title = form.get("title", [None])[0]
        template = form.get("template", [None])[0]

        if not title:
            return 0, _("No event title specified.")

        if not template:
            return 0, _("No page template specified.")

        return self.create_event(self.request)

    def render_success(self, msg, msgtype=None):

        """
        Render neither 'msg' nor 'msgtype' since redirection should occur
        instead.
        NOTE: msgtype is optional because MoinMoin 1.5.x does not support it.
        """

        pass

    def create_event(self, request):

        "Create an event page using the 'request'."

        _ = request.getText
        form = request.form

        category_pagenames = form.get("category", [])
        description = form.get("description", [None])[0]

        try:
            title = form["title"][0]
            template = form["template"][0]
            parent = form["parent"][0]

            start_day = self._get_input(form, "start-day", 0)
            start_month = self._get_input(form, "start-month", 0)
            start_year = self._get_input(form, "start-year", 0)
            end_day = self._get_input(form, "end-day", start_day)
            end_month = self._get_input(form, "end-month", start_month)
            end_year = self._get_input(form, "end-year", start_year)

            start_year, start_month, start_day = self.constrain_date(start_year, start_month, start_day)
            end_year, end_month, end_day = self.constrain_date(end_year, end_month, end_day)

        except (TypeError, ValueError):
            return 0, _("Days and years must be numbers yielding a valid date!")

        except (KeyError, IndexError):
            return 0, _("Event title or template missing.")

        start_date = self.make_date_string(start_year, start_month, start_day)
        end_date = self.make_date_string(end_year, end_month, end_day)

        # An elementary date ordering check.

        if start_date > end_date:
            start_date, end_date = end_date, start_date

        event_details = {
            "start" : start_date, "end" : end_date,
            "title" : title, "summary" : title,
            "description" : description
            }

        # Copy the template.

        template_page = PageEditor(request, template)

        if not template_page.exists():
            return 0, _("Event template not available.")

        # Use any parent page information.

        full_title = EventAggregatorSupport.getFullPageName(parent, title)

        # Load the new page and replace the event details in the body.

        new_page = PageEditor(request, full_title)

        if new_page.exists():
            return 0, _("The specified page already exists. Please choose another name.")

        # Complete the new page.

        EventAggregatorSupport.fillEventPageFromTemplate(template_page,
            new_page, event_details, category_pagenames)

        # Redirect and return success.

        query = {'action' : 'edit'}
        request.http_redirect(new_page.url(request, query))
        return 1, None

    def make_date_string(self, year, month, day):
        return "%s-%s-%s" % (
            year and ("%04d" % year) or "YYYY",
            month and ("%02d" % month) or "MM",
            day and ("%02d" % day) or "DD"
            )

    def constrain_date(self, year, month, day):
        if year and month and day:
            wd, last_day = calendar.monthrange(year, month)
            day = max(min(day, last_day), 1)
        return year, month, day

# Action function.

def execute(pagename, request):
    EventAggregatorNewEvent(pagename, request).render()

# vim: tabstop=4 expandtab shiftwidth=4
